import os
from typing import List, Tuple

from oelint_parser.cls_item import Variable
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule


class VarOverride(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.var.override',
                         severity='error',
                         message='<FOO>')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        _all: List[Variable] = list(x for x in stash.GetItemsFor(filename=_file, classifier=Variable.CLASSIFIER))  # noqa: C400
        for v in {x.VarName for x in _all}:
            items = [x for x in _all if x.VarName == v and x.IsImmediateModify()]
            items = sorted(items, key=lambda x: x.Line, reverse=False)
            for sub in [x.SubItem for x in items]:
                # Get all entries but not the only that do immediate expansion,
                # as these will be handled during parse time
                # and apply to different rules
                _items = [x for x in items if x.SubItem == sub and not x.IsAppend() and x.VarOp.strip() not in [
                    ':=', '?=', '??=']]
                if len(_items) > 1:
                    _files = {os.path.basename(x.Origin) for x in _items}
                    res += self.finding(_items[0].Origin, _items[0].InFileLine,
                                        'Variable \'{a}\' is set by \'{b}\''.format(
                                        a=v, b=','.join(_files)))
        return res



from typing import List, Tuple

from oelint_parser.cls_item import Variable
from oelint_parser.cls_stash import Stash
from oelint_parser.constants import CONSTANTS
from oelint_parser.rpl_regex import RegexRpl

from oelint_adv.cls_rule import Rule, Classification


class VarPnBpnUsage(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.vars.specific',
                         severity='error',
                         message='\'{a}\' is set specific to [\'{b}\'], but isn\'t known from PACKAGES, MACHINE, DISTRO or resources',
                         run_on=[Classification.RECIPE, Classification.BBCLASS])

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items: List[Variable] = stash.GetItemsFor(filename=_file, classifier=Variable.CLASSIFIER)
        _comp = ''.join(x.VarValueStripped for x in stash.Reduce(items, classifier=Variable.CLASSIFIER, attribute=Variable.ATTR_VAR,
                                                                 attributeValue='COMPATIBLE_MACHINE'))
        _packages = stash.GetValidPackageNames(_file)
        _named_res = stash.GetValidNamedResources(_file)
        _machines = CONSTANTS.MachinesKnown
        _distros = CONSTANTS.DistrosKnown
        _builtin_classes = ['class-native', 'class-target', 'class-nativesdk', 'class-cross']
        _operations = ['append', 'prepend', 'remove']
        for i in items:
            subs = [(x, stash.ExpandTerm(_file, x)) for x in i.SubItems]
            for subitem in subs:
                sub, expanded = subitem
                if (expanded in _builtin_classes) or (sub in _builtin_classes):
                    continue  # pragma: nocover_3.9 - coverage looks buggy on 3.9
                if (expanded in _packages) or (sub in _packages):
                    continue  # pragma: nocover_3.9 - coverage looks buggy on 3.9
                if (expanded in _operations) or (sub in _operations):
                    continue  # pragma: nocover_3.9 - coverage looks buggy on 3.9
                if (expanded in _distros) or (sub in _distros):
                    continue  # pragma: nocover_3.9 - coverage looks buggy on 3.9
                if (expanded in _named_res) or (sub in _named_res):
                    continue  # pragma: nocover_3.9 - coverage looks buggy on 3.9
                if (expanded in _machines) or (sub in _machines):
                    continue  # pragma: nocover_3.9 - coverage looks buggy on 3.9
                if _comp and (RegexRpl.match(_comp, expanded) or RegexRpl.match(_comp, sub)):
                    continue  # pragma: nocover_3.9 - coverage looks buggy on 3.9
                if expanded.startswith('task-'):
                    continue
                res += self.finding(i.Origin, i.InFileLine,
                                    override_msg=self.Msg.format(a=i.VarName, b=sub))
        return res



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsLicenseFilePrefix(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.licfileprefix'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'LIC_FILES_CHKSUM = "file://${S}/LICENSE;md5=a4a2bbea1db029f21b3a328c7a059172"',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.licfileprefix'])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'LIC_FILES_CHKSUM = "file://${S}/LICENSE;md5=a4a2bbea1db029f21b3a328c7a059172"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'LIC_FILES_CHKSUM = "file://${S}LICENSE;md5=a4a2bbea1db029f21b3a328c7a059172"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     LIC_FILES_CHKSUM = "\\
                                         file://${S}/LICENSE;md5=a4a2bbea1db029f21b3a328c7a059172 \\
                                         file://${S}/COPYING;md5=a4a2bbea1db029f21b3a328c7a059172 \\
                                     "
                                     ''',
                                 },
                             ],
                             )
    def test_fix(self, input_, id_):
        self.fix_and_check(self._create_args_fix(input_), id_)

    @pytest.mark.parametrize('id_', ['oelint.vars.licfileprefix'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'LIC_FILES_CHKSUM = "file://LICENSE;md5=a4a2bbea1db029f21b3a328c7a059172"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'LIC_FILES_CHKSUM = "file://${WORKDIR}/LICENSE;md5=a4a2bbea1db029f21b3a328c7a059172"',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import json

import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintFileUpstreamStatusOcc(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Pending'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb': 'SRC_URI = "file://test.patch"',
                                     'files/test.patch': 'Upstream-Status: Pending',
                                 },
                             ],
                             )
    def test_pending(self, input_, id_, occurrence):
        _rule_file = self._create_tempfile('rules.json', json.dumps({id_: 'warning'}))
        self.check_for_id(self._create_args(input_, ['--rulefile=' + _rule_file]), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Pending'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Pending',
                                 },
                             ],
                             )
    def test_pending_std(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Pending'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb': 'SRC_URI = "file://test.patch"',
                                     'files/test.patch': 'Upstream-Status: SomethingElse',
                                 },
                             ],
                             )
    def test_pending_no_hit(self, input_, id_, occurrence):
        _rule_file = self._create_tempfile('rules.json', json.dumps({id_: 'warning'}))
        self.check_for_id(self._create_args(input_, ['--rulefile=' + _rule_file]), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Submitted'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb': 'SRC_URI = "file://test.patch"',
                                     'files/test.patch': 'Upstream-Status: Submitted',
                                 },
                             ],
                             )
    def test_submitted(self, input_, id_, occurrence):
        _rule_file = self._create_tempfile('rules.json', json.dumps({id_: 'warning'}))
        self.check_for_id(self._create_args(input_, ['--rulefile=' + _rule_file]), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Submitted'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Submitted',
                                 },
                             ],
                             )
    def test_submitted_std(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Submitted'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb': 'SRC_URI = "file://test.patch"',
                                     'files/test.patch': 'Upstream-Status: SomethingElse',
                                 },
                             ],
                             )
    def test_submitted_no_hit(self, input_, id_, occurrence):
        _rule_file = self._create_tempfile('rules.json', json.dumps({id_: 'warning'}))
        self.check_for_id(self._create_args(input_, ['--rulefile=' + _rule_file]), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Accepted'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb': 'SRC_URI = "file://test.patch"',
                                     'files/test.patch': 'Upstream-Status: Accepted',
                                 },
                             ],
                             )
    def test_accepted(self, input_, id_, occurrence):
        _rule_file = self._create_tempfile('rules.json', json.dumps({id_: 'warning'}))
        self.check_for_id(self._create_args(input_, ['--rulefile=' + _rule_file]), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Accepted'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Accepted',
                                 },
                             ],
                             )
    def test_accepted_std(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Accepted'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb': 'SRC_URI = "file://test.patch"',
                                     'files/test.patch': 'Upstream-Status: SomethingElse',
                                 },
                             ],
                             )
    def test_accepted_no_hit(self, input_, id_, occurrence):
        _rule_file = self._create_tempfile('rules.json', json.dumps({id_: 'warning'}))
        self.check_for_id(self._create_args(input_, ['--rulefile=' + _rule_file]), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Denied'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb': 'SRC_URI = "file://test.patch"',
                                     'files/test.patch': 'Upstream-Status: Denied',
                                 },
                             ],
                             )
    def test_denied(self, input_, id_, occurrence):
        _rule_file = self._create_tempfile('rules.json', json.dumps({id_: 'warning'}))
        self.check_for_id(self._create_args(input_, ['--rulefile=' + _rule_file]), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Denied'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Denied',
                                 },
                             ],
                             )
    def test_denied_std(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Denied'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb': 'SRC_URI = "file://test.patch"',
                                     'files/test.patch': 'Upstream-Status: SomethingElse',
                                 },
                             ],
                             )
    def test_denied_no_hit(self, input_, id_, occurrence):
        _rule_file = self._create_tempfile('rules.json', json.dumps({id_: 'warning'}))
        self.check_for_id(self._create_args(input_, ['--rulefile=' + _rule_file]), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Backport'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb': 'SRC_URI = "file://test.patch"',
                                     'files/test.patch': 'Upstream-Status: Backport',
                                 },
                             ],
                             )
    def test_backport(self, input_, id_, occurrence):
        _rule_file = self._create_tempfile('rules.json', json.dumps({id_: 'warning'}))
        self.check_for_id(self._create_args(input_, ['--rulefile=' + _rule_file]), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Backport'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Backport',
                                 },
                             ],
                             )
    def test_backport_std(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Backport'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb': 'SRC_URI = "file://test.patch"',
                                     'files/test.patch': 'Upstream-Status: SomethingElse',
                                 },
                             ],
                             )
    def test_backport_no_hit(self, input_, id_, occurrence):
        _rule_file = self._create_tempfile('rules.json', json.dumps({id_: 'warning'}))
        self.check_for_id(self._create_args(input_, ['--rulefile=' + _rule_file]), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Inappropriate'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb': 'SRC_URI = "file://test.patch"',
                                     'files/test.patch': 'Upstream-Status: Inappropriate [foo]',
                                 },
                             ],
                             )
    def test_inappropriate(self, input_, id_, occurrence):
        _rule_file = self._create_tempfile('rules.json', json.dumps({id_: 'warning'}))
        self.check_for_id(self._create_args(input_, ['--rulefile=' + _rule_file]), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Inappropriate'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inappropriate [foo]',
                                 },
                             ],
                             )
    def test_inappropriate_std(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Inappropriate'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb': 'SRC_URI = "file://test.patch"',
                                     'files/test.patch': 'Upstream-Status: SomethingElse',
                                 },
                             ],
                             )
    def test_inappropriate_no_hit(self, input_, id_, occurrence):
        _rule_file = self._create_tempfile('rules.json', json.dumps({id_: 'warning'}))
        self.check_for_id(self._create_args(input_, ['--rulefile=' + _rule_file]), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Inactive-Upstream'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb': 'SRC_URI = "file://test.patch"',
                                     'files/test.patch': 'Upstream-Status: Inactive-Upstream [foo]',
                                 },
                             ],
                             )
    def test_inactive(self, input_, id_, occurrence):
        _rule_file = self._create_tempfile('rules.json', json.dumps({id_: 'warning'}))
        self.check_for_id(self._create_args(input_, ['--rulefile=' + _rule_file]), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Inactive-Upstream'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inactive-Upstream [foo]',
                                 },
                             ],
                             )
    def test_inactive_std(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus_occurance.Inactive-Upstream'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb': 'SRC_URI = "file://test.patch"',
                                     'files/test.patch': 'Upstream-Status: SomethingElse',
                                 },
                             ],
                             )
    def test_inactive_no_hit(self, input_, id_, occurrence):
        _rule_file = self._create_tempfile('rules.json', json.dumps({id_: 'warning'}))
        self.check_for_id(self._create_args(input_, ['--rulefile=' + _rule_file]), id_, occurrence)



import argparse
import hashlib
import os
import pickle  # noqa: S403
import shutil
from typing import Any, List, Tuple, Union

__default_cache_dir = os.path.join(os.environ.get('HOME'), '.oelint/caches')


class Caches():

    def __init__(self, args: argparse.Namespace) -> None:
        self.__directory = args.cachedir
        self.__enabled = args.cached
        self.__quiet = args.quiet
        self.__arg_fingerprint = self.__calculate_args_fingerprint(args)

        if self.__enabled:
            os.makedirs(self.__directory, exist_ok=True)

    @property
    def FingerPrint(self):
        return self.__arg_fingerprint

    def __calculate_args_fingerprint(self, args: argparse.Namespace) -> object:
        _hash = hashlib.sha1(usedforsecurity=False)
        for item in [
                # list all hashing relevant arguments
                args.suppress,
                args.relpaths,
                args.state.hide,
                args.release,
                args.color,
        ]:
            _hash.update(f'{item}'.encode())
        return _hash.hexdigest()

    def AddToFingerPrint(self, input_: Union[str, bytes]) -> None:
        _hash = hashlib.sha1(self.__arg_fingerprint.encode(), usedforsecurity=False)  # noqa: DUO130
        if isinstance(input_, str):
            input_ = input_.encode()
        _hash.update(input_)
        self.__arg_fingerprint = _hash.hexdigest()

    def ClearCaches(self) -> None:
        shutil.rmtree(self.__directory, ignore_errors=True)

    def GetFromCache(self, rule_ids: List[str], stash_fingerprint: str) -> Union[None, Tuple[Tuple[str, int], List[str], str]]:
        if not self.__enabled:
            return None
        _hash = hashlib.sha1(f'{self.__arg_fingerprint}{rule_ids}{stash_fingerprint}'.encode(), usedforsecurity=False)  # noqa: DUO130
        _hash_path = os.path.join(self.__directory, _hash.hexdigest())
        try:
            with open(_hash_path, 'rb') as i:  # pragma: no cover
                if not self.__quiet:
                    print(f'Using cached item {_hash_path}')
                return pickle.load(i)  # noqa: DUO103, S301
        except (pickle.PickleError, FileNotFoundError):
            return None

    def SaveToCache(self, rule_ids: List[str], stash_fingerprint: str, content: Any) -> None:
        if not self.__enabled:
            return
        _hash = hashlib.sha1(f'{self.__arg_fingerprint}{rule_ids}{stash_fingerprint}'.encode(), usedforsecurity=False)  # noqa: DUO130
        _hash_path = os.path.join(self.__directory, _hash.hexdigest())
        try:
            with open(_hash_path, 'wb') as o:
                return pickle.dump(content, o, protocol=0)
        except (pickle.PicklingError, FileNotFoundError):  # pragma: no cover
            pass  # pragma: no cover



import fnmatch
from typing import List, Tuple

from oelint_parser.cls_item import Variable
from oelint_parser.cls_stash import Stash
from oelint_parser.rpl_regex import RegexRpl

from oelint_adv.cls_rule import Rule, Classification


class VarsLayerConf(Rule):
    def __init__(self) -> None:
        self.good = [
            r'BBFILES',
            r'BBFILES_DYNAMIC',
            r'BBFILE_COLLECTIONS',
            r'BBFILE_PATTERN_.*',
            r'BBFILE_PRIORITY_.*',
            r'BBPATH',
            r'HOSTTOOLS_NONFATAL',
            r'LAYERDEPENDS_.*',
            r'LAYERRECOMMENDS_.*',
            r'LAYERSERIES_COMPAT_.*',
            r'LAYERVERSION_.*',
            r'LICENSE_PATH',
        ]

        super().__init__(id='oelint.vars.layerconf',
                         severity='warning',
                         run_on=[Classification.LAYERCONF],
                         message='{var} should not be set as part of a layer configuration')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items: List[Variable] = stash.GetItemsFor(filename=_file, classifier=Variable.CLASSIFIER,
                                                  attribute=Variable.ATTR_VAR)

        for i in items:
            if not fnmatch.fnmatch(i.Origin, '*/layer.conf'):
                continue
            if not any(RegexRpl.match(x, i.VarName) for x in self.good):
                res += self.finding(_file, i.InFileLine,
                                    self.Msg.format(var=i.VarName))
        return res



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarLicenseRemoteFile(TestBaseClass):

    def _generate_code(self, var, feature, suffix=''):
        return {'oelint_test.bb': f'{var}{suffix} = "{feature}"'}

    @pytest.mark.parametrize('id_', ['oelint.var.badimagefeature'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('var', ["IMAGE_FEATURES", "EXTRA_IMAGE_FEATURES"])
    @pytest.mark.parametrize('feature', ['allow-empty-password',
                                         'allow-root-login',
                                         'dbg-pkgs',
                                         'debug-tweaks',
                                         'dev-pkgs',
                                         'eclipse-debug',
                                         'empty-root-password',
                                         'post-install-logging',
                                         'ptest-pkgs',
                                         'serial-autologin-root',
                                         'staticdev-pkgs',
                                         'tools-debug'])
    def test_bad(self, var, feature, id_, occurrence):
        self.check_for_id(self._create_args(self._generate_code(var, feature)), f'{id_}.{feature}', occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.badimagefeature'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('var', ["IMAGE_FEATURES", "EXTRA_IMAGE_FEATURES"])
    @pytest.mark.parametrize('feature', ['weston',
                                         'x11',
                                         'x11-base',
                                         'x11-sato',
                                         'tools-profile',
                                         'tools-testapps',
                                         'tools-sdk',
                                         'nfs-server',
                                         'nfs-client',
                                         'ssh-server-dropbear',
                                         'ssh-server-openssh',
                                         'hwcodecs',
                                         'package-management',
                                         'lic-pkgs',
                                         'doc-pkgs',
                                         'bash-completion-pkgs',
                                         'read-only-rootfs',
                                         'stateless-rootfs',
                                         'splash'])
    def test_good(self, var, feature, id_, occurrence):
        self.check_for_id(self._create_args(self._generate_code(var, feature)), f'{id_}.{feature}', occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.badimagefeature'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('var', ["IMAGE_FEATURES", "EXTRA_IMAGE_FEATURES"])
    @pytest.mark.parametrize('feature', ['allow-empty-password',
                                         'allow-root-login',
                                         'dbg-pkgs',
                                         'debug-tweaks',
                                         'dev-pkgs',
                                         'eclipse-debug',
                                         'empty-root-password',
                                         'post-install-logging',
                                         'ptest-pkgs',
                                         'serial-autologin-root',
                                         'staticdev-pkgs',
                                         'tools-debug'])
    def test_good_remove(self, var, feature, id_, occurrence):
        self.check_for_id(self._create_args(self._generate_code(
            var, feature, suffix=':remove')), f'{id_}.{feature}', occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsOverrideAppend(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.overrideappend'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A:class-target:append = " a"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A:class-target:prepend = "a"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A:qemux86-64:prepend = "a"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.overrideappend'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES:${PN}-ptest:append:class-target = " a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'ALTERNATIVE:${PN}:append = " xxd"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     B:class-target = ""
                                     B:class-target:append = " a"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     B:class-target:qemux86-64 = ""
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     FILES:${PN} = "a"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



from typing import List, Tuple

from oelint_parser.cls_item import Inherit
from oelint_parser.cls_stash import Stash
from oelint_parser.parser import INLINE_BLOCK
from oelint_parser.rpl_regex import RegexRpl

from oelint_adv.cls_rule import Rule


class VarImproperInherit(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.var.improperinherit',
                         severity='error',
                         message='\'{inherit}\' is not a proper bbclass name')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items = stash.GetItemsFor(filename=_file, classifier=Inherit.CLASSIFIER)
        for i in items:
            for subi in [stash.ExpandTerm(_file, x) for x in i.get_items() if x and x != INLINE_BLOCK]:
                if not RegexRpl.match(r'^[A-Za-z0-9_.-]+$', subi):
                    res += self.finding(i.Origin, i.InFileLine,
                                        self.Msg.replace('{inherit}', subi))
        return res



import pytest  # noqa: I900

from .base import TestBaseClass


# flake8: noqa S101 - n.a. for test files
class TestClassInlineSuppressions(TestBaseClass):

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     # nooelint: oelint.vars.insaneskip
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 },
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     #   nooelint: oelint.vars.insaneskip
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 },
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     # nooelint:    oelint.vars.insaneskip
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 },
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     # nooelint: oelint.vars.insaneskip - some explanation
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 }
                             ],
                             )
    def test_inlinesuppressions_single(self, input_):
        self.check_for_id(self._create_args(input_),
                          'oelint.vars.insaneskip', 0)

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     # nooelint: oelint.vars.someother
                                     INSANE_SKIP:${PN} = "foo"
                                     ''',
                                 }
                             ],
                             )
    def test_inlinesuppressions_single_notmatching(self, input_):
        self.check_for_id(self._create_args(input_),
                          'oelint.vars.insaneskip', 1)

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     # nooelint: oelint.vars.someother
                                     INSANE_SKIP_${PN} = "foo"
                                     ''',
                                 }
                             ],
                             )
    def test_inlinesuppressions_single_notmatching_old(self, input_):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']),
                          'oelint.vars.insaneskip', 1)

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                    # nooelint: oelint.var.suggestedvar.BUGTRACKER,oelint.var.mandatoryvar.SUMMARY,oelint.var.bbclassextend
                                    INSANE_SKIP_${PN} = "foo"
                                    ''',
                                 },
                             ],
                             )
    def test_inlinesuppressions_line_1(self, input_):
        self.check_for_id(self._create_args(input_),
                          'oelint.var.suggestedvar.BUGTRACKER', 0)
        self.check_for_id(self._create_args(input_),
                          'oelint.var.mandatoryvar.SUMMARY', 0)
        self.check_for_id(self._create_args(input_),
                          'oelint.var.bbclassextend', 0)

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     # nooelint: oelint.var.badimagefeature.allow-empty-password,oelint.var.badimagefeature.allow-root-login,oelint.var.badimagefeature.empty-root-password
                                     IMAGE_FEATURES:append = " \
                                        allow-empty-password allow-root-login empty-root-password \
                                     "
                                     ''',
                                 },
                             ],
                             )
    def test_inlinesuppressions_line_2(self, input_):
        self.check_for_id(self._create_args(input_),
                          'oelint.var.badimagefeature', 0)

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                A = "1"
                                # nooelint: oelint.vars.specific,oelint.vars.doublemodify,oelint.vars.overrideappend
                                SOMEUNKNOWN_VAR:unknown-override:append += "foo"
                                ''',
                                 }
                             ],
                             )
    def test_inlinesuppressions_multiple(self, input_):
        self.check_for_id(self._create_args(input_), 'oelint.vars.specific', 0)
        self.check_for_id(self._create_args(input_),
                          'oelint.vars.doublemodify', 0)
        self.check_for_id(self._create_args(input_),
                          'oelint.vars.overrideappend', 0)

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                            SOMEUNKNOWN_VAR:unknown-override:append += "foo"
                            # nooelint: oelint.vars.specific,oelint.vars.doublemodify,oelint.vars.overrideappend
                            SOMEOTHER_VAR:unknown-override:append += "foo"
                            ''',
                                 }
                             ],
                             )
    def test_inlinesuppressions_multiple_scope(self, input_):
        self.check_for_id(self._create_args(input_), 'oelint.vars.specific', 1)
        self.check_for_id(self._create_args(input_),
                          'oelint.vars.doublemodify', 1)
        self.check_for_id(self._create_args(input_),
                          'oelint.vars.overrideappend', 1)

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     # nooelint: oelint.var.badimagefeature.allow-empty-password,,oelint.var.badimagefeature.allow-root-login,oelint.var.badimagefeature.empty-root-password
                                     IMAGE_FEATURES:append = " \
                                        allow-empty-password allow-root-login empty-root-password \
                                     "
                                     ''',
                                 },
                             ],
                             )
    def test_inlinesuppressions_remove_empty(self, input_):
        self.check_for_id(self._create_args(input_),
                          'oelint.var.badimagefeature', 0)

    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint adv-test.bb':
                                     '''
                                     # nooelint: oelint.var.badimagefeature.allow-empty-password - this is a comment
                                     IMAGE_FEATURES:append = " \
                                        allow-empty-password \
                                     "
                                     ''',
                                 },
                             ],
                             )
    def test_inlinesuppressions_with_comment(self, input_):
        self.check_for_id(self._create_args(input_),
                          'oelint.var.badimagefeature', 0)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsSRCURICHECKSUM(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurichecksum'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "ftp://foo;name=f3"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "ftp://foo;name=f3"
                                     SRC_URI[f1.md5sum] = "a"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "ftp://foo;name=f3"
                                     SRC_URI += "ftp://foo;name=f2"
                                     SRC_URI[f3.sha256sum] = "a"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "ftp://foo"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcurichecksum'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "file://foo"
                                     SRC_URI += "bzr://foo"
                                     SRC_URI += "crcc://foo"
                                     SRC_URI += "cvs://foo"
                                     SRC_URI += "ftp://foo;name=f3"
                                     SRC_URI += "git://foo;name"
                                     SRC_URI += "gitsm://foo"
                                     SRC_URI += "gitannex://foo"
                                     SRC_URI += "hg://foo"
                                     SRC_URI += "http://foo;name=f1"
                                     SRC_URI += "https://foo;name=f2"
                                     SRC_URI += "osc://foo"
                                     SRC_URI += "p4://foo"
                                     SRC_URI += "repo://foo"
                                     SRC_URI += "ssh://foo"
                                     SRC_URI += "s3://foo;name=f5"
                                     SRC_URI += "sftp://foo;name=f4"
                                     SRC_URI += "npm://foo"
                                     SRC_URI += "svn://foo"
                                     SRC_URI += "az://foo;name=f6"
                                     SRC_URI[f1.md5sum] = "a"
                                     SRC_URI[f1.sha256sum] = "a"
                                     SRC_URI[f2.md5sum] = "a"
                                     SRC_URI[f2.sha256sum] = "a"
                                     SRC_URI[f3.md5sum] = "a"
                                     SRC_URI[f3.sha256sum] = "a"
                                     SRC_URI[f4.md5sum] = "a"
                                     SRC_URI[f4.sha256sum] = "a"
                                     SRC_URI[f5.md5sum] = "a"
                                     SRC_URI[f5.sha256sum] = "a"
                                     SRC_URI[f6.md5sum] = "a"
                                     SRC_URI[f6.sha256sum] = "a"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "file://foo"
                                     SRC_URI += "bzr://foo"
                                     SRC_URI += "crcc://foo"
                                     SRC_URI += "cvs://foo"
                                     SRC_URI += "ftp://foo;sha256sum=a"
                                     SRC_URI += "git://foo;name"
                                     SRC_URI += "gitsm://foo"
                                     SRC_URI += "gitannex://foo"
                                     SRC_URI += "hg://foo"
                                     SRC_URI += "http://foo;sha256sum=a"
                                     SRC_URI += "https://foo;sha256sum=a"
                                     SRC_URI += "osc://foo"
                                     SRC_URI += "p4://foo"
                                     SRC_URI += "repo://foo"
                                     SRC_URI += "ssh://foo"
                                     SRC_URI += "s3://foo;sha256sum=a"
                                     SRC_URI += "sftp://foo;sha256sum=a"
                                     SRC_URI += "npm://foo"
                                     SRC_URI += "svn://foo"
                                     SRC_URI += "az://foo;sha256sum=a"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "ftp://foo \\
                                                 ftp://foo;name=f1"
                                     SRC_URI[md5sum] = "a"
                                     SRC_URI[sha256sum] = "a"
                                     SRC_URI[f1.md5sum] = "a"
                                     SRC_URI[f1.sha256sum] = "a"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "${@["", "file://init.cfg"][(d.getVar(\'VIRTUAL-RUNTIME_init_manager\') == \'busybox\')]}"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "http://foo;name=f1"
                                     SRC_URI[f1.sha256sum] = "a"
                                     SRC_URI[f1.md5sum] = "a"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "ftp://foo"
                                     SRC_URI[md5sum] = "a"
                                     SRC_URI[sha256sum] = "a"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "https://foo;name=f2"
                                     SRC_URI[f2.sha256sum] = "a"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "ftp://foo"
                                     SRC_URI[sha256sum] = "a"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



from typing import List, Tuple

from oelint_parser.cls_stash import Stash
from oelint_parser.cls_item import AddPylib

from oelint_adv.cls_rule import Rule, Classification


class AddpyLib(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.var.addpylib',
                         severity='error',
                         run_on=[Classification.BBCLASS, Classification.BBAPPEND, Classification.RECIPE],
                         message='addpylib is only valid in .conf files')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items_pylib: List[AddPylib] = stash.GetItemsFor(filename=_file, classifier=AddPylib.CLASSIFIER)
        for item in items_pylib:
            if not item.Origin.endswith('.conf'):  # pragma: no cover
                res += self.finding(item.Origin, item.InFileLine)
        return res



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintFuncSpecific(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.func.specific'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     do_install:append:fooarch() {
                                         abc
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     do_configure:bararch() {
                                         abc
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     COMPATIBLE_MACHINE = "xyz"
                                     do_install:fooarch() {
                                         abc
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     COMPATIBLE_MACHINE = "xyz"
                                     do_configure:append:bararch() {
                                         abc
                                     }
                                     ''',
                                 },

                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.func.specific'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     do_install_append_fooarch() {
                                         abc
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     COMPATIBLE_MACHINE = "xyz"
                                     do_install_append_bararch() {
                                         abc
                                     }
                                     ''',
                                 },

                             ],
                             )
    def test_bad_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.func.specific'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     do_install:ptest () {
                                         cp -r ${B}/testsuite ${D}${PTEST_PATH}/
                                         cp ${B}/.config      ${D}${PTEST_PATH}/
                                         ln -s /bin/busybox   ${D}${PTEST_PATH}/busybox
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     do_install:qemuall () {
                                         cp -r ${B}/testsuite ${D}${PTEST_PATH}/
                                         cp ${B}/.config      ${D}${PTEST_PATH}/
                                         ln -s /bin/busybox   ${D}${PTEST_PATH}/busybox
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     COMPATIBLE_MACHINE = "foo"
                                     do_install:append:fooarch() {
                                         abc
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     COMPATIBLE_MACHINE += "|bar"
                                     do_configure:append:bararch() {
                                         abc
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     pkg_preinst:${PN} () {
                                         abc
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     do_configure:nodistro() {
                                         abc
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     A = "nodistro"
                                     do_configure:${A} () {
                                         abc
                                     }
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.func.specific'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     do_install_ptest () {
                                         cp -r ${B}/testsuite ${D}${PTEST_PATH}/
                                         cp ${B}/.config      ${D}${PTEST_PATH}/
                                         ln -s /bin/busybox   ${D}${PTEST_PATH}/busybox
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     do_install_qemuall () {
                                         cp -r ${B}/testsuite ${D}${PTEST_PATH}/
                                         cp ${B}/.config      ${D}${PTEST_PATH}/
                                         ln -s /bin/busybox   ${D}${PTEST_PATH}/busybox
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     COMPATIBLE_MACHINE = "foo"
                                     do_install_append_fooarch() {
                                         abc
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     COMPATIBLE_MACHINE += "|bar"
                                     do_configure_append_bararch() {
                                         abc
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint-adv_test.bb':
                                     '''
                                     pkg_preinst_${PN} () {
                                         abc
                                     }
                                     ''',
                                 },
                             ],
                             )
    def test_good_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintSpacesEmptyLine(TestBaseClass):
    @pytest.mark.parametrize('id_', ['oelint.spaces.emptyline'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A = "1"
                                      
                                     B = "1"
                                     ''',  # noqa: W293 - this is exactly what we want to test
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintFileUpstreamStatusInactiveUpstreamDetails(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.file.inactiveupstreamdetails'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inactive-Upstream',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inactive-Upstream [1234]',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.inactiveupstreamdetails'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inactive-Upstream [lastcommit 1234]',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inactive-Upstream [lastrelease 1234]',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inactive-Upstream [lastcommit 1234 lastrelease 1234]',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     '''
                                     Some line before
                                     Upstream-Status: Inactive-Upstream [lastcommit 1234 lastrelease 1234]
                                     Some line after
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



from typing import List, Tuple

from oelint_parser.cls_item import Variable, Inherit
from oelint_parser.cls_stash import Stash
from oelint_parser.constants import CONSTANTS

from oelint_adv.cls_rule import Rule, Classification


class VarQuoted(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.vars.bbvars',
                         severity='warning',
                         run_on=[Classification.BBAPPEND, Classification.BBCLASS, Classification.RECIPE],
                         message='Variable \'{VAR}\' should be set on a disto/layer or local.conf level, not in a recipe',
                         appendix=CONSTANTS.GetByPath('variables/protected'))

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items: List[Variable] = stash.GetItemsFor(filename=_file,
                                                  classifier=Variable.CLASSIFIER,
                                                  attribute=Variable.ATTR_VAR,
                                                  attributeValue=CONSTANTS.GetByPath('variables/protected'))
        inherits: List[Inherit] = stash.GetItemsFor(filename=_file,
                                                    classifier=Inherit.CLASSIFIER,
                                                    attribute=Inherit.ATTR_STATEMENT,
                                                    attributeValue="INHERIT")

        for i in items:
            if i.Origin in stash.GetConfFiles():
                continue  # pragma: no cover
            if i.VarOp.strip() not in ['??=', '?=']:
                res += self.finding(i.Origin, i.InFileLine, override_msg=self.Msg.replace(
                    '{VAR}', i.VarName), appendix=i.VarName)
        for i in inherits:
            if i.Origin in stash.GetConfFiles():
                continue  # pragma: no cover
            res += self.finding(i.Origin, i.InFileLine, override_msg=self.Msg.replace(
                '{VAR}', 'INHERIT'), appendix='INHERIT')
        return res



import os

import pytest  # noqa: I900

from .base import TestBaseClass

# flake8: noqa S101 - n.a. for test files


class TestConstantFile(TestBaseClass):

    THISDIR = os.path.dirname(__file__)
    RECIPE_DISTRO_IMPORTS = os.path.join(THISDIR, "testlayer/recipes/test_known_distro_imports.bb")
    RECIPE_DISTRO = os.path.join(THISDIR, "testlayer/recipes/test_known_distro.bb")
    RECIPE_MACHINE_IMPORTS = os.path.join(THISDIR, "testlayer/recipes/test_known_machine_imports.bb")
    RECIPE_MACHINE = os.path.join(THISDIR, "testlayer/recipes/test_known_machine.bb")
    RECIPE_VARIABLE_IMPORTS = os.path.join(THISDIR, "testlayer/recipes/test_known_vars_imports.bb")
    RECIPE_VARIABLE = os.path.join(THISDIR, "testlayer/recipes/test_known_vars.bb")

    def test_distro_imports_default_file(self):
        # Test the default
        _args = self._create_args_existing_file(TestConstantFile.RECIPE_DISTRO_IMPORTS, 
                                  ['--release=scarthgap'])

        self.check_for_id(_args, 'oelint.vars.specific', 0)

    def test_distro_default_file(self):
        # Test the default
        _args = self._create_args_existing_file(TestConstantFile.RECIPE_DISTRO, 
                                  ['--release=scarthgap'])

        self.check_for_id(_args, 'oelint.vars.specific', 0)
    
    def test_machine_imports_default_file(self):
        # Test the default
        _args = self._create_args_existing_file(TestConstantFile.RECIPE_MACHINE_IMPORTS, 
                                  ['--release=scarthgap'])

        self.check_for_id(_args, 'oelint.vars.specific', 0)

    def test_machine_default_file(self):
        # Test the default
        _args = self._create_args_existing_file(TestConstantFile.RECIPE_MACHINE, 
                                  ['--release=scarthgap'])

        self.check_for_id(_args, 'oelint.vars.specific', 0)

    def test_variable_imports_default_file(self):
        # Test the default
        _args = self._create_args_existing_file(TestConstantFile.RECIPE_VARIABLE_IMPORTS, 
                                  ['--release=scarthgap'])

        self.check_for_id(_args, 'oelint.vars.mispell.unknown', 0)

    def test_variable_default_file(self):
        # Test the default
        _args = self._create_args_existing_file(TestConstantFile.RECIPE_VARIABLE, 
                                  ['--release=scarthgap'])

        self.check_for_id(_args, 'oelint.vars.mispell.unknown', 0)

    def test_distro_imports_specific_file(self):
        # Test the default
        _args = self._create_args_existing_file(TestConstantFile.RECIPE_DISTRO_IMPORTS, 
                                  ['--release=styhead'])

        self.check_for_id(_args, 'oelint.vars.specific', 1)

    
    def test_machine_imports_specific_file(self):
        # Test the default
        _args = self._create_args_existing_file(TestConstantFile.RECIPE_MACHINE_IMPORTS, 
                                  ['--release=styhead'])

        self.check_for_id(_args, 'oelint.vars.specific', 1)

    def test_variable_imports_specific_file(self):
        # Test the default
        _args = self._create_args_existing_file(TestConstantFile.RECIPE_VARIABLE_IMPORTS, 
                                  ['--release=styhead'])

        self.check_for_id(_args, 'oelint.vars.mispell.unknown', 1)


import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsOutOfContextConf(TestBaseClass):

    def __generate_sample_code(self, var):
        return '''
            {var} = "foo"
            '''.format(var=var)

    @pytest.mark.parametrize('id_', ['oelint.vars.outofcontext'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('var', [
        "FILESEXTRAPATHS",
    ])
    @pytest.mark.parametrize('filename', [
        'test_.bb',
        'classes/test.bbclass',
        'conf/machine/test.conf',
        'conf/layer.conf',
        'conf/distro/test.conf',
    ])
    def test_bad(self, id_, var, filename, occurrence):
        input_ = {
            filename: self.__generate_sample_code(var),
        }
        self.check_for_id(self._create_args(input_, extraopts=['--mode=all']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.outofcontext'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('var', [
        "FILESEXTRAPATHS",
    ])
    @pytest.mark.parametrize('filename', [
        'test_%.bbappend',
    ])
    def test_good(self, id_, var, filename, occurrence):
        input_ = {
            filename: self.__generate_sample_code(var),
        }
        self.check_for_id(self._create_args(input_, extraopts=['--mode=all']), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsAutorev(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.doublesemicolon'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('var', [
        "LIC_FILES_CHKSUM",
        "SRC_URI",
    ])
    def test_bad(self, var, id_, occurrence):
        input_ = {
            'test.bb': f'{var} = "foo;;bar;;;;baz;;a=1;b=2"\n',
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.doublesemicolon'])
    @pytest.mark.parametrize('var', [
        "LIC_FILES_CHKSUM",
        "SRC_URI",
    ])
    def test_fix(self, var, id_):
        input_ = {
            'test.bb': f'{var} = "foo;;bar;;;;baz;;a=1;b=2"\n',
        }
        self.fix_and_check(self._create_args_fix(input_), id_)

    @pytest.mark.parametrize('id_', ['oelint.vars.doublesemicolon'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('var', [
        "LIC_FILES_CHKSUM",
        "SRC_URI",
    ])
    def test_good(self, var, id_, occurrence):
        input_ = {
            'test.bb': f'{var} = "foo;bar;baz;a=1;b=2"\n',
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900
from oelint_parser.constants import CONSTANTS

from .base import TestBaseClass


class TestClassOelintVarsOutOfContextConf(TestBaseClass):

    def __generate_sample_code(self, var, extra=''):
        return '''
            {extra}
            {var} = "foo"
            '''.format(var=var, extra=extra)

    @pytest.mark.parametrize('id_', ['oelint.vars.outofcontext'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('var', [
        "EXTRA_IMAGE_FEATURES",
        "IMAGE_FEATURES",
    ])
    @pytest.mark.parametrize('extra', [
        '',
        'inherit something',
    ])
    def test_bad(self, id_, var, extra, occurrence):
        input_ = {
            'test.bb': self.__generate_sample_code(var, extra),
        }
        self.check_for_id(self._create_args(input_, extraopts=['--mode=all']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.outofcontext'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('var', [
        "EXTRA_IMAGE_FEATURES",
        "IMAGE_FEATURES",
    ])
    @pytest.mark.parametrize('extra', [
        *[f'inherit {x}' for x in CONSTANTS.ImagesClasses],
    ])
    def test_good(self, id_, var, extra, occurrence):
        input_ = {
            'test.bb': self.__generate_sample_code(var, extra),
        }
        self.check_for_id(self._create_args(input_, extraopts=['--mode=all']), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarLicenseSPDX(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.licensesdpx'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     LICENSE:a = "ISC &MIT"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     LICENSE:e = "(ISC|MIT) & Apache-2.0 & BSD-3-Clause"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.licensesdpx'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     LICENSE:${PN} = "BSD-2-Clause"
                                     LICENSE:b = "(ISC | MIT)"
                                     LICENSE:c = "(BSD-2-Clause | MIT)"
                                     LICENSE:d = "(BSD-2-Clause | MIT) & MIT"
                                     LICENSE:f = "BSD-3-Clause"
                                     LICENSE = "Unknown"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsSRCURIdomains(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.srcuridomains'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "git://abc.group.com/a.git"
                                     SRC_URI += "git://def.group.com/b.git"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI:class-devupstream = "git://foo.bar;branch=master;protocol=https"
                                     SRC_URI:append:class-devupstream = " git://foo.baz;branch=master;protocol=https"
                                     SRC_URI:remove:class-devupstream:qemuall = "git://foo.baz;branch=master;protocol=https"
                                     SRC_URI = "git://foo.baz;branch=master;protocol=https"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI:class-devupstream = "git://foo.bar;branch=master;protocol=https"
                                     SRC_URI:append:class-devupstream = " git://foo.baz;branch=master;protocol=https"
                                     SRC_URI = "git://foo.baz;branch=master;protocol=https"
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.srcuridomains'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "git://abc.group.com/a.git"
                                     SRC_URI += "ftp://abc.group.com/some.patch"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "git://abc.group.com/a.git"
                                     SRC_URI += "file://some.patch"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI += "ftp://abc.group.com/some.patch"
                                     SRC_URI += "${@["", "file://init.cfg"][(d.getVar('VIRTUAL-RUNTIME_init_manager') == 'busybox')]}"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI:class-devupstream = "git://foo.bar;branch=master;protocol=https"
                                     SRC_URI:remove:class-devupstream = "git://foo.baz;branch=master;protocol=https"
                                     SRC_URI = "git://foo.baz;branch=master;protocol=https"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI:class-devupstream = "git://foo.bar;branch=master;protocol=https"
                                     SRC_URI:remove:class-devupstream = "git://foo.bar;branch=master;protocol=https"
                                     SRC_URI = "git://foo.baz;branch=master;protocol=https"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI:class-devupstream = "git://foo.bar;branch=master;protocol=https"
                                     SRC_URI = "git://foo.baz;branch=master;protocol=https"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI:class-devupstream = "git://foo.baz;branch=master;protocol=https"
                                     SRC_URI = "git://foo.baz;branch=master;protocol=https"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     SRC_URI:append:class-devupstream = " git://foo.baz;branch=master;protocol=https"
                                     SRC_URI = "git://foo.baz;branch=master;protocol=https"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarInheritLast(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.var.inheritlast.native'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit native
                                     inherit foo
                                     ''',
                                 },
                             ],
                             )
    def test_bad_inherit_native(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.inheritlast.nativesdk'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit nativesdk
                                     inherit foo
                                     ''',
                                 },
                             ],
                             )
    def test_bad_inherit_nativesdk(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.inheritlast.cross'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit cross
                                     inherit foo
                                     ''',
                                 },
                             ],
                             )
    def test_bad_inherit_cross(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.inheritlast',
                                     'oelint.var.inheritlast.native',
                                     'oelint.var.inheritlast.nativesdk',
                                     'oelint.var.inheritlast.cross'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit abc native
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit abc nativesdk
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit abc cross
                                     ''',
                                 },
                             ],
                             )
    def test_good_inherit(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsDoubleModify(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.doublemodify'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'A:append:prepend:remove += "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A:append:prepend += "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A:append += "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A:prepend:remove += "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A:append:remove += "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A:remove += "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A:append:prepend:remove = " 1 "',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A:append:remove = " 1 "',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A:prepend:remove = " 1 "',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.doublemodify'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'A_append_prepend_remove += "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A_append_prepend += "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A_append += "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A_prepend_remove += "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A_append_remove += "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A_remove += "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A_append_prepend_remove = " 1 "',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A_append_remove = " 1 "',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A_prepend_remove = " 1 "',
                                 },
                             ],
                             )
    def test_bad_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.doublemodify'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'A:append = " a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A:prepend = "a "',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A:remove = "a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A += "a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A =+ "a"',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.doublemodify'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'A_append = " a"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A_prepend = "a "',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A_remove = "a"',
                                 },
                             ],
                             )
    def test_good_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarFilesOverride(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.var.filesoverride'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES:${PN} = " foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES:${PN} := "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES:${PN}-dev = "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES:${PN}-dev := "foo"',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.filesoverride'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES_${PN} = " foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES_${PN} := "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES_${PN}-dev = "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILES_${PN}-dev := "foo"',
                                 },
                             ],
                             )
    def test_bad_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.filesoverride'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES:SOLIBSDEV = "abc"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES:${PN}:append = " foo"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES:${PN}:prepend = "foo "',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES:${PN} += "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES:${PN} =+ "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES:${PN} .= " foo"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES:${PN} =. "foo "',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES:${PN}-dev += "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES:${PN}-dev =+ "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES:${PN}-dev .= " foo"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES:${PN}-dev =. "foo "',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.filesoverride'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES_SOLIBSDEV = "abc"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES_${PN}_append = " foo"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES_${PN}_prepend = "foo "',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES_${PN} += "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES_${PN} =+ "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES_${PN} .= " foo"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES_${PN} =. "foo "',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES_${PN}-dev += "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES_${PN}-dev =+ "foo"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES_${PN}-dev .= " foo"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILES_${PN}-dev =. "foo "',
                                 },
                             ],
                             )
    def test_good_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsInAppClassOverride(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.inappclassoverride'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'A:class-native = "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'A:class-nativesdk = "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A:class-native = "1"
                                     BBCLASSEXTEND = "nativesdk"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A:class-nativesdk = "1"
                                     BBCLASSEXTEND = "native"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_a:append:class-native() {
                                        :
                                     }
                                     BBCLASSEXTEND = "nativesdk"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_a:append:class-nativesdk() {
                                        :
                                     }
                                     BBCLASSEXTEND = "native"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_a:append:class-native() {
                                        :
                                     }
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_a:append:class-nativesdk() {
                                        :
                                     }
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.inappclassoverride'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A:class-native = "1"
                                     inherit native
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A:class-nativesdk = "1"
                                     inherit nativesdk
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A:class-native = "1"
                                     BBCLASSEXTEND = "native"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     A:class-nativesdk = "1"
                                     BBCLASSEXTEND = "nativesdk"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bbclass':
                                     'A:class-native = "1"',
                                 },
                                 {
                                     'oelint_adv_test.bbclass':
                                     'A:class-nativesdk = "1"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'A:class-native = "1"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'A:class-nativesdk = "1"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_a:append:class-native() {
                                        :
                                     }
                                     BBCLASSEXTEND = "native"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_a:append:class-nativesdk() {
                                        :
                                     }
                                     BBCLASSEXTEND = "nativesdk"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_a:append:class-native() {
                                        :
                                     }
                                     inherit native
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     do_a:append:class-nativesdk() {
                                        :
                                     }
                                     inherit nativesdk
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



from typing import List, Tuple

from oelint_parser.cls_item import Inherit
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule, Classification


class VarNativeSDKFilename(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.var.nativesdkfilename',
                         severity='warning',
                         run_on=[Classification.BBAPPEND, Classification.RECIPE],
                         message='nativesdk-recipe-files should include \'nativesdk-\' in file name')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items: List[Inherit] = [x for x in
                                stash.GetItemsFor(filename=_file, classifier=Inherit.CLASSIFIER)
                                if 'nativesdk' in x.get_items()]
        if any(items):
            if _file.find('nativesdk-') == -1:
                res += self.finding(_file, 0)
        return res



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsLayerConf(TestBaseClass):

    def __generate_sample_code(self, var, operation):
        return '''
            {var} {operation} "foo"
            '''.format(var=var, operation=operation)

    @pytest.mark.parametrize('id_', ['oelint.vars.layerconf'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('var', [
        'BBFILES',
        'BBFILES_DYNAMIC',
        'BBFILE_COLLECTIONS',
        'BBFILE_PATTERN_foo',
        'BBFILE_PRIORITY_foo',
        'BBPATH',
        'HOSTTOOLS_NONFATAL',
        'LAYERDEPENDS_foo',
        'LAYERRECOMMENDS_foo',
        'LAYERSERIES_COMPAT_foo',
        'LAYERVERSION_foo',
        'LICENSE_PATH',
    ])
    @pytest.mark.parametrize('operation', ['=', ':=', '.=', '=.', '+=', '=+', ' =+'])
    def test_good(self, id_, var, operation, occurrence):
        input_ = {
            'conf/layer.conf': self.__generate_sample_code(var, operation),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.layerconf'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('var', [
        'BBFILES',
        'BBFILES_DYNAMIC',
        'BBFILE_COLLECTIONS',
        'BBFILE_PATTERN_foo',
        'BBFILE_PRIORITY_foo',
        'BBPATH',
        'HOSTTOOLS_NONFATAL',
        'LAYERDEPENDS_foo',
        'LAYERRECOMMENDS_foo',
        'LAYERSERIES_COMPAT_foo',
        'LAYERVERSION_foo',
        'LICENSE_PATH',
    ])
    @pytest.mark.parametrize('operation', ['=', ':=', '.=', '=.', '+=', '=+', ' =+'])
    def test_good_non_layer(self, id_, var, operation, occurrence):
        input_ = {
            'oelint-test_1.0.bb': self.__generate_sample_code(var, operation),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.layerconf'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('var', [
        'SOME_OTHER_VAR',
    ])
    @pytest.mark.parametrize('operation', ['=', ':=', '.=', '=.', '+=', '=+', ' =+'])
    def test_bad(self, id_, var, operation, occurrence):
        input_ = {
            'conf/layer.conf': self.__generate_sample_code(var, operation),
        }
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarMultiInherit(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.var.multiinherit'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'inherit abc abc',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit def
                                     inherit ghi def
                                     ''',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.var.multiinherit'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     inherit abc def
                                     inherit autotools \\
                                             gobject-introspection \\
                                             pkgconfig
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsFileSettingsDouble(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.filessetting.double'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     FILES:${PN} += "${bindir}"
                                     FILES:${PN}-ping = "${base_bindir}/ping.${BPN}"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     '''
                                     FILES:${PN} += "${bindir}"
                                     FILES:${PN}-ping = "${base_bindir}/ping.${BPN}"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     FILES:${PN} += "${bindir}"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     FILES:${PN}-doc += "${docdir}"
                                     ''',
                                 },

                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.filessetting.double'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     FILES_${PN} += "${bindir}"
                                     FILES_${PN}-ping = "${base_bindir}/ping.${BPN}"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     '''
                                     FILES_${PN} += "${bindir}"
                                     FILES_${PN}-ping = "${base_bindir}/ping.${BPN}"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     FILES_${PN} += "${bindir}"
                                     ''',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     FILES_${PN}-doc += "${docdir}"
                                     ''',
                                 },
                             ],
                             )
    def test_bad_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.filessetting.double'])
    @pytest.mark.parametrize('occurrence', [2])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     FILES:${PN} += "/opt/other/path"
                                     FILES:${PN}-ping = "${base_bindir}/ping.${BPN}"
                                     FILES:${PN} += "/opt/other/path"
                                     ''',
                                 },
                             ],
                             )
    def test_bad_non_default(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.filessetting.double'])
    @pytest.mark.parametrize('occurrence', [2])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     FILES_${PN} += "/opt/other/path"
                                     FILES_${PN}-ping = "${base_bindir}/ping.${BPN}"
                                     FILES_${PN} += "/opt/other/path"
                                     ''',
                                 },
                             ],
                             )
    def test_bad_non_default_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.filessetting.double'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     FILES:${PN} += "/opt/other/path"
                                     FILES:${PN}-ping = "${base_bindir}/ping.${BPN}"
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.filessetting.double'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     '''
                                     FILES_${PN} += "/opt/other/path"
                                     FILES_${PN}-ping = "${base_bindir}/ping.${BPN}"
                                     ''',
                                 },
                             ],
                             )
    def test_good_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)



import os
from typing import List, Tuple

from oelint_parser.cls_item import Variable
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule, Classification


class FilePatchIsSignedOff(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.file.patchsignedoff',
                         severity='warning',
                         run_on=[Classification.BBAPPEND, Classification.RECIPE],
                         message='Patch \'{FILE}\' should contain a Signed-off-by entry')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        items = stash.GetFiles(_file, '*.patch')
        _items = stash.GetItemsFor(filename=_file, classifier=Variable.CLASSIFIER,
                                   attribute=Variable.ATTR_VAR, attributeValue='SRC_URI')
        for i in items:
            with open(i) as _input:
                try:
                    content = _input.readlines()
                    if not any(x for x in content if x.startswith('Signed-off-by: ')):
                        # Find matching SRC_URI assignment
                        _assign = [x for x in _items if x.VarValue.find(
                            os.path.basename(i)) != -1]
                        if any(_assign):  # pragma: no cover
                            res += self.finding(_assign[0].Origin,
                                                _assign[0].InFileLine,
                                                self.Msg.replace('{FILE}', os.path.basename(i)))
                except UnicodeDecodeError:  # pragma: no cover
                    pass  # pragma: no cover
        return res



import os
from typing import List, Tuple

import regex  # noqa: I900
from oelint_parser.cls_item import Variable
from oelint_parser.cls_stash import Stash
from oelint_parser.rpl_regex import RegexRpl

from oelint_adv.cls_rule import Rule, Classification


class FilePatchIsUpstreamStatusInAppMsg(Rule):
    def __init__(self):
        super().__init__(id='oelint.file.inappropriatemsg',
                         severity='info',
                         run_on=[Classification.BBAPPEND, Classification.RECIPE],
                         message='Patch \'{FILE}\' with Upstream-Status Inappropriate has to have a reasoning appended in [], chosen from {choices}')

    def _get_recipe(self, items, path):
        # Find matching SRC_URI assignment
        return [x for x in items if x.VarValue.find(os.path.basename(path)) != -1]

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        patches = stash.GetFiles(_file, '*.patch')
        _items = stash.GetItemsFor(filename=_file, classifier=Variable.CLASSIFIER,
                                   attribute=Variable.ATTR_VAR, attributeValue='SRC_URI')

        _valid_classifiers = [
            'oe-specific',
            'OE specific',
            'oe-core specific',
            'not author',
            'native',
            'licensing',
            'configuration',
            'enable feature',
            'disable feature',
            'bugfix .*',
            'embedded specific',
            'no upstream',
            'other',
        ]

        _valid_class = {
            'Inappropriate': r'Inappropriate\s+\[({a})\]'.format(a='|'.join(_valid_classifiers)),
        }

        for i in patches:
            with open(i) as _input:
                _recipe_match = self._get_recipe(_items, i)
                if _recipe_match:
                    _recipe_match = _recipe_match[0]
                else:
                    continue  # pragma: no cover
                try:
                    for m in RegexRpl.finditer(r'^Upstream-Status:\s*(?P<class>.*)', _input.read(), flags=regex.regex.MULTILINE):
                        for k, v in _valid_class.items():
                            if m.group('class').strip().startswith(k) and not RegexRpl.match(v, m.group('class')):
                                res += self.finding(_recipe_match.Origin,
                                                    _recipe_match.InFileLine,
                                                    self.Msg.format(FILE=os.path.basename(i), choices=','.join(_valid_classifiers)))
                except UnicodeDecodeError:  # pragma: no cover
                    pass  # pragma: no cover
        return res



from typing import List, Tuple

from oelint_parser.cls_item import Variable, Inherit, Function
from oelint_parser.cls_stash import Stash

from oelint_adv.cls_rule import Rule, Classification


class VarsInappClassOverride(Rule):
    def __init__(self) -> None:
        super().__init__(id='oelint.vars.inappclassoverride',
                         severity='warning',
                         run_on=[Classification.RECIPE],
                         message='{var} is using {override}, but the recipe does not set {class_} in BBCLASSEXTEND or inherits the class {class_}')

    def check(self, _file: str, stash: Stash) -> List[Tuple[str, int, str]]:
        res = []
        inherits = set()
        for item in stash.GetItemsFor(filename=_file, classifier=Inherit.CLASSIFIER):
            inherits.update(item.get_items())
        bbclassextend = stash.ExpandVar(_file, Variable.ATTR_VAR, attributeValue='BBCLASSEXTEND').get('BBCLASSEXTEND', [])

        for item in stash.GetItemsFor(filename=_file, classifier=[Variable.CLASSIFIER, Function.CLASSIFIER]):
            for needle in [('class-native', 'native'), ('class-nativesdk', 'nativesdk')]:
                override, class_ = needle
                if override not in item.SubItems:
                    continue
                if class_ in bbclassextend or class_ in inherits:
                    continue
                name = item.FuncName if isinstance(item, Function) else item.VarName
                res += self.finding(item.Origin, item.InFileLine,
                                    override_msg=self.Msg.format(var=name, override=override, class_=class_))
        return res



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintFileUpstreamStatus(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'This is not a patch',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Acceppted',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Submit',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inapropriate',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.file.upstreamstatus'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Accepted',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Backport',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Denied',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inappropriate [bugfix this is a bugfix]',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inappropriate [configuration]',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inappropriate [disable feature]',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inappropriate',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inappropriate [embedded specific]',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inappropriate [enable feature]',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inappropriate [licensing]',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inappropriate [native]',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inappropriate [no upstream]',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inappropriate [other]',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Pending',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Submitted [http://some.where]',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inactive-Upstream',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inactive-Upstream [lastcommit 1234]',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inactive-Upstream [lastrelease 1234]',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     'Upstream-Status: Inactive-Upstream [lastcommit 1234 lastrelease 1234]',
                                 },
                                 {
                                     'oelint_adv-test.bb':
                                     'SRC_URI = "file://test.patch"',
                                     'files/test.patch':
                                     '''
                                     Some line before that
                                     Upstream-Status: Inappropriate
                                     Some line after that
                                     ''',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)



import pytest  # noqa: I900

from .base import TestBaseClass


class TestClassOelintVarsFilextrapaths(TestBaseClass):

    @pytest.mark.parametrize('id_', ['oelint.vars.fileextrapaths'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'FILESEXTRAPATHS:prepend := "${THISDIR}/file"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILESEXTRAPATHS:append := "${THISDIR}/file"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILESEXTRAPATHS += "${THISDIR}/file"',
                                 },
                             ],
                             )
    def test_bad(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.fileextrapaths'])
    @pytest.mark.parametrize('occurrence', [1])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bb':
                                     'FILESEXTRAPATHS_prepend := "${THISDIR}/file"',
                                 },
                                 {
                                     'oelint_adv_test.bb':
                                     'FILESEXTRAPATHS_append := "${THISDIR}/file"',
                                 },
                             ],
                             )
    def test_bad_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.fileextrapaths'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILESEXTRAPATHS:prepend := "${THISDIR}/file"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILESEXTRAPATHS:append := "${THISDIR}/file"',
                                 },
                             ],
                             )
    def test_good(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_), id_, occurrence)

    @pytest.mark.parametrize('id_', ['oelint.vars.fileextrapaths'])
    @pytest.mark.parametrize('occurrence', [0])
    @pytest.mark.parametrize('input_',
                             [
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILESEXTRAPATHS_prepend := "${THISDIR}/file"',
                                 },
                                 {
                                     'oelint_adv_test.bbappend':
                                     'FILESEXTRAPATHS_append := "${THISDIR}/file"',
                                 },
                             ],
                             )
    def test_good_old(self, input_, id_, occurrence):
        self.check_for_id(self._create_args(input_, ['--release=dunfell']), id_, occurrence)
